require 'console'
require 'logging'
require 'dotenv/load'
require 'fileutils'
require 'eventmachine'
require 'sqlite3'
require 'rufus/scheduler'
require 'ostruct'
require 'yaml'
require 'to_regexp'
require 'xmlsimple'
require 'pry'
require 'pastel'

module Rsrb
  autoload :Server,             'rsrb/server'

  module Content
    module PM
      autoload :Presence,         'rsrb/content/pm'
    end
  end


  module Internal
    autoload :Utils,                'rsrb/internal/utils'

    module Types
      autoload :Routine,            'rsrb/internal/types/routine'
    end
  end

  module Engine
    autoload :EventManager,     'rsrb/core/engine'
    autoload :Event,            'rsrb/core/engine'
    autoload :QueuePolicy,      'rsrb/core/engine'
    autoload :WalkablePolicy,   'rsrb/core/engine'
    autoload :Action,           'rsrb/core/engine' # TODO move to Actions
    autoload :ActionQueue,      'rsrb/core/engine' # TODO move to Actions
  end

  module Misc
    autoload :AutoHash,            'rsrb/core/util'
    autoload :HashWrapper,         'rsrb/core/util'
    autoload :Flags,               'rsrb/core/util'
    autoload :TextUtils,           'rsrb/core/util'
    autoload :NameUtils,           'rsrb/core/util'
    autoload :ThreadPool,          'rsrb/core/util'
    autoload :Cache,               'rsrb/core/cache'
  end

  module Actions
    autoload :HarvestingAction,    'rsrb/core/actions'
  end

  module Model
    autoload :Equipment,           'rsrb/model/equipment'
    autoload :HitType,             'rsrb/model/combat'
    autoload :Hit,                 'rsrb/model/combat'
    autoload :Damage,              'rsrb/model/combat'
    autoload :Animation,           'rsrb/model/effects'
    autoload :Graphic,             'rsrb/model/effects'
    autoload :ChatMessage,         'rsrb/model/effects'
    autoload :Entity,              'rsrb/model/entity'
    autoload :Location,            'rsrb/model/location'
    autoload :Player,              'rsrb/model/player'
    autoload :RegionManager,       'rsrb/model/region'
    autoload :Region,              'rsrb/model/region'
  end

  module Item
    autoload :Item,                       'rsrb/model/item'
    autoload :ItemDefinition,             'rsrb/model/item'
    autoload :Container,                  'rsrb/model/item'
    autoload :ContainerListener,          'rsrb/model/item'
    autoload :InterfaceContainerListener, 'rsrb/model/item'
    autoload :WeightListener,             'rsrb/model/item'
    autoload :BonusListener,              'rsrb/model/item'
  end

  module NPC
    autoload :NPC,                 'rsrb/model/npc'
    autoload :NPCDefinition,       'rsrb/model/npc'
  end

  module Player
    autoload :Appearance,          'rsrb/model/player/appearance'
    autoload :InterfaceState,      'rsrb/model/player/interfacestate'
    autoload :RequestManager,      'rsrb/model/player/requestmanager'
    autoload :Skills,              'rsrb/model/player/skills'
  end

  module Net
    autoload :ActionSender,        'rsrb/net/actionsender'
    autoload :ISAAC,               'rsrb/net/isaac'
    autoload :Session,             'rsrb/net/session'
    autoload :Connection,          'rsrb/net/connection'
    autoload :Packet,              'rsrb/net/packet'
    autoload :PacketBuilder,       'rsrb/net/packetbuilder'
    autoload :JaggrabConnection,   'rsrb/net/jaggrab'
  end

  module GroundItems
    autoload :GroundItem,          'rsrb/services/ground_items'
    autoload :GroundItemEvent,     'rsrb/services/ground_items'
    autoload :PickupItemAction,    'rsrb/services/ground_items'
  end

  module Shops
    autoload :ShopManager,         'rsrb/services/shops'
    autoload :Shop,                'rsrb/services/shops'
  end

  module Objects
    autoload :ObjectManager,       'rsrb/services/objects'
  end

  module Doors
    autoload :DoorManager,         'rsrb/services/doors'
    autoload :Door,                'rsrb/services/doors'
    autoload :DoubleDoor,          'rsrb/services/doors'
    autoload :DoorEvent,           'rsrb/services/doors'
  end

  module Tasks
    autoload :NPCTickTask,         'rsrb/tasks/npc_update'
    autoload :NPCResetTask,        'rsrb/tasks/npc_update'
    autoload :NPCUpdateTask,       'rsrb/tasks/npc_update'
    autoload :PlayerTickTask,      'rsrb/tasks/player_update'
    autoload :PlayerResetTask,     'rsrb/tasks/player_update'
    autoload :PlayerUpdateTask,    'rsrb/tasks/player_update'
    autoload :SystemUpdateEvent,   'rsrb/tasks/sysupdate_event'
    autoload :UpdateEvent,         'rsrb/tasks/update_event'
  end

  module World
    autoload :Constants,           'rsrb/world/constants'
    autoload :Profile,             'rsrb/world/profile'
    autoload :Pathfinder,          'rsrb/world/walking'
    autoload :Point,               'rsrb/world/walking'
    autoload :Instance,               'rsrb/world/world'
    autoload :PlayerFileLoader,      'rsrb/world/loader'
    autoload :NPCSpawns,           'rsrb/world/npc_spawns'
    autoload :ItemSpawns,          'rsrb/world/item_spawns'
    autoload :Task,                'rsrb/world/task'
  end


end

require 'rsrb/plugin_hooks'
require 'rsrb/net/packetloader'

include Rsrb::Internal::Utils