module Rsrb::Internal::Utils

  # Gets this class' name.
  def class_name
    self.class.name.split('::').last.to_sym
  end

  ##
  # Initialize loggers.
  def init_loggers
    return unless ENV['LOGGING'].to_i.positive?

    FileUtils.mkdir_p("#{FileUtils.pwd}/assets/log")
    FileUtils.mkdir_p("#{FileUtils.pwd}/assets/log/err")
    file_names = %W[#{FileUtils.pwd}/assets/log/#{class_name}-#{Time.now.strftime('%Y-%m-%d-%s').chomp}.log
                    #{FileUtils.pwd}/assets/log/err/#{class_name}-#{Time.now.strftime('%Y-%m-%d-%s').chomp}.log]
    file_names.each { |f| File.write(f, '') }
    @e_logfile = Logger.new(file_names[0])
    @logfile = Logger.new(file_names[1])
    @pastel = Pastel.new if ENV['LESS_BORING'].to_i > 1
    @log = Console.logger
  rescue StandardError => e
    puts 'An error occurred while initializing loggers!'
    puts e
  end

  ##
  # Log info lines
  # @param lines [Array] lines of text that are passed to the logger.
  def log(*lines)
    return unless ENV['LOGGING'].to_i.positive?

    lines.each do |line|
      @logfile.info(line)
      if @pastel
        @log.info(@pastel.green("[#{Time.now.strftime('[%H:%M:%S')}]:[#{class_name}] ~> #{line}"))
      else
        @log.info("[#{Time.now.strftime('[%H:%M:%S')}]:[#{class_name}] ~> #{line}")
      end
    end
    nil
  end

  ##
  # Log warning lines
  # @param lines [Array] lines of text that are passed to the logger.
  def log!(*lines)
    return unless ENV['LOGGING'].to_i.positive?

    lines.each do |line|
      @logfile.warn(line)
      if @pastel
        @log.warn(@pastel.yellow("[#{Time.now.strftime('[%H:%M:%S')}]:[#{class_name}] ~> #{line}"))
      else
        @log.warn("[#{Time.now.strftime('[%H:%M:%S')}]:[#{class_name}] ~> #{line}")
      end
    end
  end

  ##
  # Log error lines
  # @param lines [Array] lines of text that are passed to the logger.
  def err(*lines)
    return unless ENV['LOGGING'].to_i.positive?

    lines.each do |line|
      @e_logfile.error(line)
      if @pastel
        @log.error(@pastel.magenta.bold("[#{Time.now.strftime('[%H:%M:%S')}]:[#{class_name}] ~> #{line}"))
      else
        @log.error("[#{Time.now.strftime('[%H:%M:%S')}]:[#{class_name}] ~> #{line}")
      end
    end
    nil
  end

  ##
  # Log fatal lines
  # @param lines [Array] lines of text that are passed to the logger.
  def err!(*lines)
    return unless ENV['LOGGING'].to_i.positive?

    lines.each do |line|
      @e_logfile.fatal(line)
      if @pastel
        @log.error(@pastel.red.bold("[#{Time.now.strftime('[%H:%M:%S')}]:[#{class_name}] ~> #{line}"))
      else
        @log.error("[#{Time.now.strftime('[%H:%M:%S')}]:[#{class_name}] ~> #{line}")
      end
    end
    nil
  end
  # Returns the file name as a symbol.
  # @param string [String] The path to the file.
  def symbolize_file(string)
    File.basename(string, '*.rb').to_sym
  end
end